#include "globalData.h"
#include "TControlFunctions.h"
#include "TTMExtraction.h"

#include "TTree.h"
#include "TSQLRow.h"
#include "TLeaf.h"
#include "TStopwatch.h"

#include <string>
#include <iostream>
#include <iomanip>
#include <stdlib.h>
#include <stdio.h>
#include <netinet/in.h>

#define FILE_COMPRESSION                2
#define DEBUG_MSG                       1

using namespace std;

ClassImp(TControlFunctions)

TControlFunctions::TControlFunctions(){
   // TControlFctions constructor.
#if (DEBUG_MSG)
   cout << "+ TControlFunctions::Constructor()" << endl;
#endif
	sema_gD->Wait();
	sqlclient = new TMySQLClient(gD.gMySqlServerHost, gD.gDatabaseName, gD.gMySqlUserName, gD.gMySqlPassword);
	//tmapclient = new TTmapClient(gD.gTmapHost, gD.gTmapPort);
	sema_gD->Post();

	treeFileInfo = NULL;
	rootFile = NULL;
}

TControlFunctions::~TControlFunctions(){
   // TControlFctions destructor.
#if (DEBUG_MSG)
   cout << "+ TControlFunctions::Destructor()" << endl;
#endif
   if(rootFile != NULL) {			
      if(!rootFile->IsZombie()) {
         if(rootFile->IsOpen()) {
            cout << "+ Write all objects and close file" << endl;
            rootFile->Write("",TObject::kOverwrite);
            rootFile->Close();
         }
         delete rootFile;
		 rootFile = NULL;
      }
   }
}
//______________________________________________________________________________
int TControlFunctions::Init() {
   // Initializing ROOT database file
#if (DEBUG_MSG)
   cout << "+ TControlfunctions::Init()" << endl;
#endif

	int rc;
	sema_gD->Wait();
	rc = InitRootFile();
	if(rc != 0) return -1;
#if (!FUNTIONAL_VERSION_2)	
	rc = InitParamTable();
	if(rc != 0) return -1;
#endif
	sema_gD->Post();
	return 0;
}
//______________________________________________________________________________
int TControlFunctions::InitRootFile() {
#if (DEBUG_MSG)
   cout << "+ TControlfunctions::InitRootFile()" << endl;
#endif
   // Create database file
   // return  0: file opened successful
   // return -1: file could not be opened
   // return -2: file info tree incurrupt or wrong version

   stringstream dbname;
   
   dbname << gD.gDatabaseName << ".root";
	
   TDatime dt;
   rootFile = new TFile(dbname.str().c_str(),"UPDATE","PaCTS TM Archive",FILE_COMPRESSION);
   
   if(rootFile->IsZombie()) {
      cout << "+ ERROR: opening file" << endl;
      return -1;
   }

   thisFileInfo.version = gD.gFileVersion;
   thisFileInfo.year    = dt.GetYear();
   thisFileInfo.month   = dt.GetMonth();
   thisFileInfo.hour    = dt.GetHour();
   thisFileInfo.minute  = dt.GetMinute();
   thisFileInfo.second  = dt.GetSecond();
   strcpy(thisFileInfo.dbname, gD.gDatabaseName.c_str());

   string treeName("file_info");
   string branchName("creation_info");

   // already exists a file information tree?
   treeFileInfo = (TTree*) rootFile->Get(treeName.c_str());
   if(treeFileInfo == NULL) {
      treeFileInfo = new TTree(treeName.c_str(),"General file information");
      treeFileInfo->Branch(branchName.c_str(), &thisFileInfo,
                        "fileVersion/F:year:month:hour:minute:second:database/C");
      treeFileInfo->Fill();
      treeFileInfo->Write();
   }
   else {
      // The tree exists! Check the file information:
      if(!treeFileInfo->GetBranchStatus(branchName.c_str())) {
         cout << "+ ERROR: file info branch not found" << endl;
         rootFile->Close();
         return -2;
      } else {
         TBranch* branch = NULL;
         branch = treeFileInfo->GetBranch(branchName.c_str());
         treeFileInfo->SetBranchAddress(branchName.c_str(), &thisFileInfo);
         treeFileInfo->GetEntry(0);

         // Check file version
         if(thisFileInfo.version != gD.gFileVersion) {
            cout << "+ ERROR: wrong file version | "
                 << "file: " << thisFileInfo.version << " "
                 << "should be: " << gD.gFileVersion
                 << endl;
            rootFile->Close();
            return -2;
         }

         // Check databasename
         if(strcmp(thisFileInfo.dbname, gD.gDatabaseName.c_str()) > 0 ||
            strcmp(thisFileInfo.dbname, gD.gDatabaseName.c_str()) < 0) {
            cout << "+ ERROR: wrong sql database | "
                 << "file: " << thisFileInfo.dbname << " "
                 << "should be: " << gD.gDatabaseName
                 << endl;
            rootFile->Close();
            return -2;
         }
      }
   }
   delete treeFileInfo; 
   treeFileInfo = NULL;
   return 0;
}
//______________________________________________________________________________
int TControlFunctions::InitParamTable() {
   // Create data structure of the ROOT file
#if (DEBUG_MSG)
   cout << "+ TControlFunctions::InitParamTable()" << endl;
#endif
   
   // get database tree from root file
   string treeName("mysql_db");
   treeSqlDb = (TTree*) rootFile->Get(treeName.c_str());

   if(treeSqlDb == NULL) {

      treeSqlDb = new TTree(treeName.c_str(), "This tree holds the parameter description table");
      
      // generate all necessary branches | printf("--> %s\n", (*fieldNames[i]).c_str());
          
           treeSqlDb->Branch("Dummy1", &paramName,"paramName/C");

           treeSqlDb->Branch("Dummy2", &paramDescript, "paramDescript/C");

           treeSqlDb->Branch("Dummy3", &pktName, "pktName/C");

           treeSqlDb->Branch("Dummy4", &wordNb, "wordNb/I");

           treeSqlDb->Branch("Dummy5", &bitOffset, "bitOffset/I");

           treeSqlDb->Branch("Dummy6", &size, "size/I");

           treeSqlDb->Branch("Dummy7", &type, "type/C");

           treeSqlDb->Branch("Dummy8", &unit, "unit/C");

           treeSqlDb->Branch("Dummy9", &apid, "apid/I");

           treeSqlDb->Branch("Dummy10", &pktId, "pktId/I");

         treeSqlDb->Fill();

      }
      // Write the branches to the tree
      treeSqlDb->Write("",TObject::kOverwrite);
   return 0;
}
//______________________________________________________________________________
void TControlFunctions::SaveRootFile(){
//save the root file.
    cout << "---> Writing Trees to File" << endl;
	rootFile->Write();
	rootFile->Flush();	
}
